
from modeltranslation.translator import register, TranslationOptions

from poll.models import Poll, PollChoice


@register(Poll)
class ArticleTranslationOptions(TranslationOptions):

    fields = ('question', )


@register(PollChoice)
class ArticleTagTranslationOptions(TranslationOptions):

    fields = ('value', )




from django.db import models
from django.db.models import Q
from django.utils.functional import cached_property
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from ordered_model.models import OrderedModelBase


class Poll(models.Model):

    question = models.CharField(_('Question'), max_length=255)

    created = models.DateTimeField(_('Created'), auto_now_add=True)

    votes = models.IntegerField(_('Votes'), default=0, editable=False)

    def is_voted(self, user, user_ip, session_key):
        check_query = Q(ip=user_ip) | Q(session=session_key)

        if user.is_authenticated():
            check_query = check_query | Q(user=user)

        return self.vote_set.filter(check_query).exists()

    def __unicode__(self):
        return self.question

    class Meta:
        get_latest_by = 'id'
        ordering = ['-id']
        verbose_name = _('Poll')
        verbose_name_plural = _('Polls')


class PollChoice(OrderedModelBase):

    poll = models.ForeignKey(
        Poll, verbose_name=_('Poll'), related_name='choices')

    value = models.CharField(_('Value'), max_length=255)

    votes = models.IntegerField(_('Votes'), default=0, editable=False)

    order = models.PositiveIntegerField(_('Ordering'), default=0)

    order_field_name = 'order'
    order_with_respect_to = 'poll'

    @cached_property
    def percent(self):
        if self.votes:
            return int(float(self.votes) / float(self.poll.votes) * 100)
        return 0

    def __unicode__(self):
        return self.value

    class Meta:
        ordering = ('order', 'id', )
        unique_together = ['poll', 'value']
        verbose_name = _('Poll choice')
        verbose_name_plural = _('Poll choices')


class Vote(models.Model):

    user = models.ForeignKey(
        get_user_model(), verbose_name=_('User'), null=True, editable=False)

    poll = models.ForeignKey(Poll, verbose_name=_('Poll'))

    choice = models.ForeignKey(PollChoice, verbose_name=_('Choice'))

    created = models.DateTimeField(
        _('Created'), editable=False, auto_now_add=True, db_index=True)

    ip = models.CharField(_('IP'), max_length=40, editable=False, db_index=True)

    session = models.CharField(
        _('Session'), max_length=40, editable=False, db_index=True)

    user_agent = models.CharField(
        _('User agent'), max_length=255, editable=False)

    def __unicode__(self):
        return self.choice.value

    class Meta:
        ordering = ['-created']
        verbose_name = _('Vote')
        verbose_name_plural = _('Votes')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-02-23 13:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('poll', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='poll',
            options={'get_latest_by': 'id', 'ordering': ['-id'], 'verbose_name': 'Poll', 'verbose_name_plural': 'Polls'},
        ),
        migrations.AlterModelOptions(
            name='pollchoice',
            options={'ordering': ('order',), 'verbose_name': 'Poll choice', 'verbose_name_plural': 'Poll choices'},
        ),
        migrations.AddField(
            model_name='pollchoice',
            name='order',
            field=models.PositiveIntegerField(db_index=True, default=1, editable=False),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='poll',
            name='created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created'),
        ),
        migrations.AlterField(
            model_name='poll',
            name='votes',
            field=models.IntegerField(default=0, editable=False, verbose_name='Votes'),
        ),
        migrations.AlterField(
            model_name='pollchoice',
            name='poll',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='choices', to='poll.Poll', verbose_name='Poll'),
        ),
        migrations.AlterField(
            model_name='pollchoice',
            name='value',
            field=models.CharField(max_length=255, verbose_name='Value'),
        ),
        migrations.AlterField(
            model_name='pollchoice',
            name='votes',
            field=models.IntegerField(default=0, editable=False, verbose_name='Votes'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='choice',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='poll.PollChoice', verbose_name='Choice'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='created',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Created'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='ip',
            field=models.CharField(db_index=True, editable=False, max_length=40, verbose_name='IP'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='poll',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='poll.Poll', verbose_name='Poll'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='session',
            field=models.CharField(db_index=True, editable=False, max_length=40, verbose_name='Session'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='user',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AlterField(
            model_name='vote',
            name='user_agent',
            field=models.CharField(editable=False, max_length=255, verbose_name='User agent'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-10-14 10:48
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Poll',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question', models.CharField(max_length=255, verbose_name='Question')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('votes', models.IntegerField(default=0, editable=False)),
            ],
            options={
                'ordering': ['-id'],
                'verbose_name': 'Poll',
                'verbose_name_plural': 'Polls',
            },
        ),
        migrations.CreateModel(
            name='PollChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=255)),
                ('votes', models.IntegerField(default=0, editable=False)),
                ('poll', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='poll.Poll')),
            ],
            options={
                'ordering': ['value'],
                'verbose_name': 'Poll choice',
                'verbose_name_plural': 'Poll choices',
            },
        ),
        migrations.CreateModel(
            name='Vote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('ip', models.CharField(db_index=True, editable=False, max_length=40)),
                ('session', models.CharField(db_index=True, editable=False, max_length=40)),
                ('user_agent', models.CharField(editable=False, max_length=255)),
                ('choice', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='poll.PollChoice')),
                ('poll', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to='poll.Poll')),
                ('user', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-created'],
                'verbose_name': 'Vote',
                'verbose_name_plural': 'Votes',
            },
        ),
        migrations.AlterUniqueTogether(
            name='pollchoice',
            unique_together=set([('poll', 'value')]),
        ),
    ]




from django import forms

from poll.models import Vote


class VoteForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(VoteForm, self).__init__(*args, **kwargs)

        poll = self.initial.get('poll')

        if poll:
            self.fields['choice'] = forms.ModelChoiceField(
                empty_label=None, queryset=poll.choices.all(),
                widget=forms.RadioSelect)

    class Meta:
        model = Vote
        fields = ['poll', 'choice']
        widgets = {
            'poll': forms.HiddenInput
        }




from django.contrib import admin

from modeltranslation.admin import TranslationAdmin, TranslationTabularInline

from poll.models import Poll, PollChoice
from suit.sortables import SortableTabularInlineBase


class PollChoiceInline(SortableTabularInlineBase, TranslationTabularInline):

    model = PollChoice
    extra = 0
    min_num = 2


class PollAdmin(TranslationAdmin):

    inlines = [PollChoiceInline]

    list_display = ['question', 'created', 'votes']


admin.site.register(Poll, PollAdmin)




from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class PollConfig(AppConfig):
    name = 'poll'
    verbose_name = _("Poll")


default_app_config = 'poll.PollConfig'

__version__ = '2.6'




from django.shortcuts import redirect, render
from django.views.generic import FormView
from django.http.response import (
    HttpResponseBadRequest, JsonResponse, HttpResponse)
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from misc.utils import get_ip, get_session_key

from poll.forms import VoteForm
from poll.models import Poll


def get_latest_poll(request):

    try:
        poll = Poll.objects.latest()
    except Poll.DoesNotExist:
        return HttpResponse(_('No poll found'))

    is_poll_voted = poll.is_voted(
        request.user, get_ip(request), get_session_key(request))

    form = VoteForm(initial={'poll': poll})

    context = {
        'poll': poll,
        'form': form,
        'is_poll_voted': is_poll_voted
    }

    return render(request, 'poll/index.html', context)


class VoteView(FormView):

    form_class = VoteForm
    http_method_names = ['post']

    def form_valid(self, form):

        request = self.request
        user = request.user
        user_ip = get_ip(request)

        vote = form.save(commit=False)

        session_key = get_session_key(request)

        poll = vote.poll

        if poll.is_voted(user, user_ip, session_key):
            return HttpResponseBadRequest('Already voted')

        vote.ip = user_ip
        vote.session = session_key
        vote.user_agent = request.META.get('HTTP_USER_AGENT', '')[:255]

        if user.is_authenticated():
            vote.user = user

        vote.save()

        vote.choice.votes += 1
        vote.choice.save()

        poll.votes += 1
        poll.save()

        if request.is_ajax():
            return JsonResponse({
                'message': _('Vote was successfully created'),
                'result': render_to_string('poll/result.html', {
                    'poll': poll,
                    'is_poll_voted': True
                })
            })

        return redirect(request.GET.get('referrer', 'home'))

    def form_invalid(self, form):
        return HttpResponseBadRequest('Invalid request')




from django.conf.urls import url

from poll import views


urlpatterns = [

    url(r'^latest/$', views.get_latest_poll, name='latest'),

    url(r'^vote/$', views.VoteView.as_view(), name='vote')

]




from django.apps import apps
from django import template

from misc.utils import get_ip, get_session_key

from ..forms import VoteForm


register = template.Library()


@register.inclusion_tag('poll/index.html', takes_context=True)
def render_latest_poll(context):

    context = context.__dict__.copy()

    request = context['request']

    poll_model = apps.get_model('poll', 'Poll')

    try:
        poll = poll_model.objects.latest()
    except poll_model.DoesNotExist:
        return {}

    is_poll_voted = poll.is_voted(
        request.user, get_ip(request), get_session_key(request))

    form = VoteForm(initial={'poll': poll})

    context.update({
        'poll': poll,
        'form': form,
        'is_poll_voted': is_poll_voted
    })

    return context
